/*
 * vt_splash.c
 * Splash a file to a VT and change to it
 */

/*
 * Copyright 2008 Roy Marples <roy@marples.name>
 * All rights reserved

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

const char cs_copyright[] = "Copyright (c) 2008 Roy Marples";

#define _BSD_SOURCE

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <linux/vt.h>

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>

#define UNCONST(a)	((void *)(unsigned long)(const void *)(a))

#define LOGO		"/etc/cursplash.logo"
#define MESSAGE		"Initializing the kernel ..."

#define SPLASH_TTY	17
#define SPLASH_TTY_PATH	"/dev/tty17"

#define HIDE_CURSOR	"\033[?25l"
#define MOVE_CURSOR	"\033[%d;%dH"

int
main(void)
{
	int tty0_fd, splash_fd;
	char buffer[BUFSIZ];
	size_t len;
	FILE *f;
	struct winsize ws;

	if ((tty0_fd = open("/dev/tty0", O_RDWR, 0)) == -1) {
		perror("open /dev/tty0");
		exit(EXIT_FAILURE);
	}
	if ((splash_fd = open(SPLASH_TTY_PATH, O_RDWR, 0)) == -1) {
		perror("open " SPLASH_TTY_PATH);
		exit(EXIT_FAILURE);
	}
	if ((f = fopen(LOGO, "r")) == NULL) {
		perror("open " LOGO);
		exit(EXIT_FAILURE);
	}
	if (ioctl(tty0_fd, VT_ACTIVATE, (void *)SPLASH_TTY) == -1) {
		perror("ioctl VT_ACTIVATE");
		exit(EXIT_FAILURE);
	}
	if (ioctl(tty0_fd, VT_WAITACTIVE, (void *)SPLASH_TTY) == -1) {
		perror("ioctl VT_WAITACTIVE");
		exit(EXIT_FAILURE);
	} 
	/* Hide the cursor */
	dup2(splash_fd, STDOUT_FILENO);
	printf(HIDE_CURSOR);
	while ((fgets(buffer, sizeof(buffer), f))) {
		len = strlen(buffer);
		write(splash_fd, buffer, len);
	}
	fclose(f);
	if (ioctl(splash_fd, TIOCGWINSZ, &ws) == -1) {
		ws.ws_col = 80;
		ws.ws_row = 25;
	}
	printf(MOVE_CURSOR "%s" MOVE_CURSOR,
	       ws.ws_row,
	       (ws.ws_col - strlen(MESSAGE)) / 2,
	       MESSAGE, 0, 0);
	return 0;
}
